const http = require('http')
const express = require('express')
const { graphqlHTTP } = require('express-graphql')
const { graphqlUploadExpress } = require('graphql-upload')
const cors = require('cors')

const { buildFederatedSchema } = require('@apollo/federation')

const { pataka: env } = require('ahau-env')()

const Main = require('@ssb-graphql/main')
const Profile = require('@ssb-graphql/profile')
const Invite = require('@ssb-graphql/invite')
const Stats = require('@ssb-graphql/stats')
const Pataka = require('@ssb-graphql/pataka')

module.exports = function graphqlServer (ssb, cb) {
  const PORT = env.graphql.port
  const app = express()
  app.options('*', cors())

  const main = Main(ssb, { type: 'pataka' })
  const profile = Profile(ssb)
  const invite = Invite(ssb)
  const stats = Stats(ssb)
  const pataka = Pataka(ssb, profile.gettersWithCache)

  if (ssb.config.graphql && ssb.config.graphql.loadContext === false) {
    return next(null)
  }

  main.loadContext(next)

  function next (err, context = {}) {
    if (err) return cb(err)

    const schema = buildFederatedSchema([
      main,
      profile,
      invite,
      pataka,
      stats
    ])

    const app = express()
    app.use(cors({
      origin (origin, cb) {
        const { NODE_ENV, PLATFORM } = process.env

        // TODO: does this need to worry about cordova?
        if (PLATFORM === 'cordova' && (origin === undefined || origin === 'file://')) return cb(null, true)

        switch (NODE_ENV) {
          case undefined: // if NODE_ENV not set, apply production, standards
          case 'production':
            if (origin === undefined) return cb(null, true) // Electron
            else if (origin === `http://localhost:${PORT}`) return cb(null, true) // GraphiQL
            else return cb(new Error('CORS issue')) // give no hints

          case 'development':
            if (origin === undefined) return cb(null, true) // Electron
            if (origin === 'http://localhost:8081') return cb(null, true) // Webpack dev server
            else if (origin === `http://localhost:${PORT}`) return cb(null, true) // GraphiQL
            else return cb(new Error(`CORS issue - unexpected header 'Origin': '${origin}'`))

          default:
            cb(new Error(`invalid NODE_ENV - ${NODE_ENV}`))
        }
      }
    }))

    const MAX_FILE_SIZE = 5 * 1024 * 1024 // 5MB
    app.use(
      '/graphql',
      graphqlUploadExpress({ maxFileSize: MAX_FILE_SIZE, maxFiles: 10 }),
      graphqlHTTP({
        schema,
        context,
        graphiql: true
      })
    )

    const httpServer = http.createServer(app)

    httpServer.listen(PORT, (err) => {
      if (err) return cb(err)

      cb(null, { port: PORT })
    })

    ssb.close.hook((close, args) => {
      httpServer.close()
      close(...args)
    })
  }
}
